@@ -111,7 +111,7 @@ extends 'Bio::Root::Root';
 
 has metacommunity => (
    is => 'rw',
-   isa => 'Bio::Community::Meta',
+   isa => 'Maybe[Bio::Community::Meta]',
    required => 0,
    lazy => 1,
    default => undef,
@@ -233,11 +233,12 @@ has by_rel_ab => (
 =cut
 
 method get_summary () {
+   # Sanity check
    my $meta = $self->metacommunity;
-
-   if ($meta->get_communities_count == 0) {
-      $self->throw("Need to provide at least one community.");
+   if ( (not $meta) || ($meta->get_communities_count == 0) ) {
+      $self->throw('Should have a metacommunity containing at least one community');
    }
+
    my $summary = $meta;
 
    # Then merge duplicates
